#!/usr/bin/env python
# -*- coding: utf-8 -*-


import sys
sys.path.insert(1, 'csdownscale/')

from tools import cut_domain 
from config import Config
from dataset import get_obs, get_hcst, get_fcst
from downscale import apply_downscale
import logging

def main(run_conf,run_sdate):
    """ Main program """

    
    conf = Config(run_conf)
    logging.basicConfig(filename=conf.logfile, level=conf.loglevel)

    conf.sdates = [run_sdate] 
    apply_to = conf.apply_to 

    for target_var in conf.target_vars:
    
        # load obs
        print(f'target_var:{target_var}')
    
        print('loading obs for analogs training...')
        targetobs, srcobs = get_obs(conf, target_var)

        print('lazy load completed...')

        for region in conf.regions.keys():

            print(f'region:{region}')

            # cut obs
            print('cutting domain and loading obs data in memory...')    
            srcobs_sub = cut_domain(srcobs, conf.regions[region]).load()
            targetobs_sub = cut_domain(targetobs, conf.regions[region]).load()
            print('obs load completed...')    


            for fcst_sdate in conf.sdates:

                print(f'fcst start date:{fcst_sdate}')

                # load exp

                if apply_to == "all" or apply_to == "hcst":

                    exp = get_hcst(conf,region,fcst_sdate,target_var)
                    apply_downscale(exp,srcobs_sub,targetobs_sub,target_var,region,conf)

                if apply_to == "all" or apply_to == "fcst":
                    fcst = get_fcst(conf,region,fcst_sdate,target_var)
                    apply_downscale(fcst,srcobs_sub,targetobs_sub,target_var,region,conf)

 
if __name__ == "__main__":

    run_conf = sys.argv[1] 
    #run_conf = "conf/vitigeoos-seasonal-run.yml"
    run_sdate = sys.argv[2]
    #run_sdate = "20210701"

    main(run_conf,run_sdate)

 