library(FNN)
library(multiApply)
library(ClimProjDiags) # for Subset
library(abind)
get_knn <- function(train, test, k=3) {
	# Reformat train and test as an array with (time, points)
	train <- aperm(Apply(train, margins = "time", fun=function(x) { as.vector(x) })$output1 )
	test <- aperm(Apply(test, margins = "time", fun=function(x) { as.vector(x) })$output1 )

	# For each row in test, find the k closest rows in train
	knn.ind <- get.knnx(train, test, k)
	return(knn.ind)
}

analogs <- function(train, test, obs, k=3, fun=NULL) {
	# Check input dims
#	if(!all(names(dim(train)) == names(dim(test)))) { stop("Parameters train and test should have the same dimension names, and in the same order") }
#	if(!any(names(dim(train)) == 'time')) { stop("Dimension 'time' not found in train") }
#	if(!any(names(dim(obs)) == 'time')) { stop("Dimension 'time' not found in obs") }
#	pos_time <- which(names(dim(train)) == 'time')
#	if(any(dim(train)[-pos_time] != dim(test)[-pos_time])) { stop("train and test should have the same dimensions except for time") }
#	if(dim(train)["time"] != dim(obs)["time"]) { stop("train and obs should have the same 'time' dimension") }

	# Get knn indices for each test element
	knn.ind <- get.knnx(train, test, k)
	idx <- knn.ind$nn.index
	names(dim(idx)) <- c("time", "analog")

	# Obtain the obs for the knn indices
	analogs <- Apply(idx, margins="analog", fun=function(an) { obs[an,,] })$output1

	# Summarize if needed
	if(!is.null(fun)) {
		# Perfrom an operation (e.g. analogs mean)
		analogs <- apply(analogs, c(1, 2, 3), fun)
	}

	#analogs <- lapply(1:nrow(knn.ind$nn.index), function(i) { Apply(Subset(obs, "time", knn.ind$nn.index[i,]), "time", fun)$output1 })
	#analogs <- Apply(idx, margins="time", fun=function(x) { apply(obs[x,,], c(2,3), mean)})

	return(list(analogs=analogs, indices=knn.ind$nn.index, distances=knn.ind$nn.dist))
}
