import numpy as np
import rpy2.robjects as robjects
from sklearn.metrics import r2_score, mean_squared_error, make_scorer
from sklearn.neighbors import KNeighborsRegressor
from sklearn.model_selection import GridSearchCV
from typing import Dict, List

X = np.asarray(robjects.r('readRDS("tas_era5.cds_southwest.europe_holdout.RDS")'))
y = np.asarray(robjects.r('readRDS("tas_mescan.remapbil_iberian.peninsula_holdout.RDS")'))

predictors = ['tas', 'psl', 'g500', 'sfcWind', 'uas', 'vas', 'prlr']
train = {}
for p in predictors:
    filename = p + '_era5.cds_southwest.europe_holdout.RDS'
    train[p] = np.asarray(robjects.r('readRDS("' + filename + '")'))

def select_features(train: Dict, select: List[str]):
    return {key: value for key, value in train.items() if key in select}

def merge_features(train: Dict):
    tmp = [val.reshape(val.shape[0],-1) for val in train.values()]
    return np.concatenate(tmp, axis=1)

def crop_region_weights(
    longitudes,
    latitudes,
    lonmin: float,
    lonmax: float,
    latmin: float,
    latmax: float
):
    array = np.outer((latmin <= latitudes) & (latitudes <= latmax), (lonmin <= longitudes) & (longitudes <= lonmax))
    return array.reshape(-1)

pipeline = Pipeline(
    [
        ("feature_select", SelectKBest(f_regression)),
        ("scale", StandardScaler()), 
        ("model", KNeighborsRegressor(random_state=0)),
    ]
)

search = RandomSearchCV(
    estimator=pipeline,
    param_grid={
        "selector__k": [3, 4, 5, 6, 7, 8, 9, 10],
        "model__n_estimators": np.arange(10, 200, 10),
    },
    n_jobs=-1,
    scoring="neg_mean_squared_error",
    cv=5,
    verbose=3,
)

X = X.reshape(2921,-1)
y = y.reshape(2921,-1)

kk = KNeighborsRegressor(n_neighbors=1).fit(X,y)
newy = kk.predict(X[0,:])
np.all(newy == y[0,:])

mean_squared_error(newy, y[0,:] , squared=False)

# default multioutput mean_squared_error returns the rmse across time for each pixel
# whereas this version returns the image rmse for each sample (time instance)
def rms_error(y_true, y_pred):
    sse = (y_true - y_pred)**2
    rmse = np.sqrt(sse.mean(axis=1))
    return rmse.mean()

neg_rmse = make_scorer(rms_error, greater_is_better=False)

params = { 'n_neighbors': np.arange(1,11,2) }
search = GridSearchCV(estimator=KNeighborsRegressor(), param_grid=params, scoring=neg_rmse)
search.fit(X,y)

search.best_params_
search.best_score_
search.cv_results_

